#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : sdiags.py
# Contents: create detection diagrams for single data set
# Author  : Christian Borgelt
# History : 2014.06.06 file created
#-----------------------------------------------------------------------
from sys     import argv, path
from os.path import join
from math    import log10
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import psts2mp

#-----------------------------------------------------------------------

#cmax = zmax = 9
cmax = zmax = 12

#-----------------------------------------------------------------------

def header (out, pro):
    out.write('verbatimtex\n')
    out.write('\\documentclass[12pt]{article}\n')
    out.write('\\usepackage{mathptmx}\n')
    out.write('\\def\\hnorm{\\hbox to0pt{\\phantom{g)}\\hss}}\n')
    out.write('\\begin{document}\n')
    out.write('etex\n\n')
    out.write('prologues := ' +str(pro) +';\n\n')
    out.write('input axes;\n')
    out.write('input barchart;\n\n')
    out.write('numeric expand;  expand = 1/3;\n')
    out.write('numeric e;       e      = 2.718281828;\n\n')
    out.write('vardef init (expr ttop, tbot) =\n')
    out.write('  numeric data[][];\n')
    out.write('  numeric zmin, zmax, n; n = 0;\n')
    out.write('  zmin = -1; zmax = 3;\n')
    out.write('  barchart.set_defaults("ptmr8r");\n')
    out.write('  barchart.clipcolor        := 0.7white;\n')
    out.write('  barchart.invalid.barcolor := 0.5white;\n')
    out.write('  barchart.invalid.max      := zmin-1.5;\n')
    out.write('  barchart.null.max         := zmin-0.5;\n')
    out.write('  barchart.set_units(30mm/'+str(cmax-1)
                                +', 30mm/'+str(zmax-1)
                                +', 24mm/(zmax-zmin));\n')
    out.write('  barchart.set_ranges(('+str(cmax-1)+',0.84,0.08), '
                                   +'('+str(zmax-1)+',0.84,0.08), '
                                   +'(zmin,zmax));\n')
    out.write('  barchart.set_view (-120, 36);\n')
    out.write('  barchart.set_light(-120, 75);\n')
    out.write('  axes.title.size       := 1.2;  % 12pt\n')
    out.write('  axes.title.skip       := 1.1 *fontsize "ptmr8r";\n')
    out.write('  axes.title.offset     := (0mm,1mm);\n')
    out.write('  axes.x.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.x.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.x.alabel.offset  := (0,4mm);\n')
    out.write('  axes.y.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.y.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.y.alabel.offset  := (0,4mm);\n')
    out.write('  axes.z.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.z.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.z.grid.thickness := 0.4pt;\n')
    out.write('  axes.z.kinked         := true;\n')
    out.write('  barchart.draw_xticks("2"')
    for i in range(3,cmax+1):
        s = str(i) if cmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
    out.write(');\n')
    out.write('  barchart.draw_yticks("2"')
    for i in range(3,zmax+1):
        s = str(i) if cmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
    out.write(');\n')
    out.write('  barchart.draw_zticks(-1,0,1,2,3);\n')
    out.write('  barchart.draw_xlabel(0, '
                  +'btex \\small pattern size~$z$\\hnorm etex);\n')
    out.write('  barchart.draw_ylabel(0, '
                  +'btex \\small coincidences~$c$\\hnorm etex);\n')
    out.write('  axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('  barchart.draw_zlabel(0, '
                  +'btex \\small '
                  +'$\\log(\\mbox{\\#patterns})$\\hnorm etex);\n')
    out.write('  barchart.draw_title(-3, -1, ttop, tbot);\n')
    out.write('  barchart.draw_frame;\n')
    out.write('enddef;\n\n')
    out.write('vardef set_data (text list) =\n')
    out.write('  barchart.set_data(data[n], list);\n')
    out.write('  n := n+1;\n')
    out.write('enddef;\n\n')

#-----------------------------------------------------------------------

def footer (out):
    out.write('end.\n')

#-----------------------------------------------------------------------

def readfile (fname):
    with open(fname, 'r') as inp:
        c = [l.split() for l in inp]
    return dict([((int(x),int(y)),float(z)) for x,y,z in c])

#-----------------------------------------------------------------------

def diag (out, fid, name, top, bot, excl):
    excl = readfile(join('data','pc_n100_p3_r20_w3_c.txt')).keys() \
           if excl else set()   # get the excluded pattern signatures
    vals = readfile(join('data',name+'.txt')) # and read data to display
    out.write('beginfig(' +str(fid) +'); % '+name+'\n')
    out.write('  init("'+top+'", "'+bot+'");\n')
    for z in range(2,zmax+1):   # traverse the group sizes (neurons)
        out.write('  set_data(')# start the next data row
        for c in range(2,cmax+1): # traverse the numbers of coincidences
            if c > 2: out.write(', ')
            x = vals[(z,c)] if (z,c) in vals else 0
            x = log10(x) if x > 0 else -2
            if (z,c) in excl: x = -3
            out.write('%f' % (x if x <= 3 else 4))
        out.write(');\n')       # print the values/fill data matrix
    out.write('  barchart.draw_data(data, 0, 5, expand);\n')
    out.write('endfig;\n\n')    # draw the diagram

#-----------------------------------------------------------------------

if __name__ == '__main__':
    fid = 0
    with open(argv[1] if len(argv) > 1 else 'special.mp', 'w') as out:
        header(out, 2)
        diag(out, fid, 'pcs_all', 'frequent', 'patterns', False)
        fid += 1
        diag(out, fid, 'pcs_flt', 'filtered', 'patterns', True)
        fid += 1
        diag(out, fid, 'pcs_red', 'reduced',  'patterns', True)
        fid += 1
        psts = dict()           # initialize the train dictionary
        with open(join('data','trains.txt'), 'rb') as inp:
            for line in inp:    # read the spike trains
                i,t = line.split()
                i,t = int(i),float(t)
                if i in psts: psts[i].append(t)
                else:         psts[i] = [t]
        psts = [psts[i] for i in sorted([i for i in psts])]
        out.write('defaultscale := 1;\n\n')
        psts2mp(out, fid, psts, 3, 72, 36, name='trains',
                xlabel='btex \\scriptsize time (3 seconds)\\hnorm etex',
                ylabel='btex \\scriptsize items/neurons\\hnorm etex')
        fid += 1
        footer(out)
