#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : single.py
# Contents: detect synchronous spiking activity and reduce results
# Author  : Christian Borgelt
# History : 2014.06.06 file created from detect/single.py
#           2015.08.12 adapted to modified pattern set reduction
#-----------------------------------------------------------------------
from sys     import argv, stderr, path
from os.path import join
from random  import seed
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import genpsts
try:
    from coco    import coconad, patred
except ImportError:
    COCODIR = join('..', 'coconad')
    if COCODIR not in path: path.insert(0, COCODIR)
    from coconad import coconad
    FIMDIR   = join('..', 'fim')
    if FIMDIR   not in path: path.insert(0, FIMDIR)
    from patred  import patred
    print('Attention: Could not import from PyCoCo library!')
    print('           Using Python replacement (very slow)!')

#-----------------------------------------------------------------------

oo = float('inf')               # positive infinity as a constant
redids = ['x', 'c', 'C', 'i', 's', 'S', 'l', 'L', 't', 'T']

#-----------------------------------------------------------------------

def single (ncnt=100, period=3, rate=20,  # neuron    parameters
            size=5, coin=5, jitter=0.003, # assembly  parameters
            width=0.003, rfnid=5):        # detection parameters
    base = 'n%d_p%g_r%g_w%g_c' % (ncnt,period,rate,width*1000)
    psp  = set()                # initialize a pattern spectrum
    with open(join('data','pc_'+base+'.txt'), 'r') as inp:
        for line in inp:        # collect the pattern signatures
            a,b,c = line.split()
            psp.add((int(a),int(b)))
    border = list(range(ncnt+1)); m = -1
    for z in reversed(border):  # determine the detection border
        m = max([c for a,c in psp if a == z]+[m])
        border[z] = m+1         # (support thresholds per size)
    border[0:2] = [oo,oo]       # entirely rule out sizes 0 and 1
    asm  = set(range(1,size+1)) # create the assembly to inject
    psts = genpsts(ncnt, period, rate, asms=[[asm,-coin]])
    eseq = [(i+1,s) for i in range(ncnt) for s in psts[i]]
    eseq.sort(key = lambda k: k[1])
    with open(join('data','trains.txt'), 'w') as out:
        for s in eseq: out.write('%d %.15g\n' % s)
    eseq = None                 # write parallel spike trains
    pats = coconad(list(zip(range(1,ncnt+1), psts)),
                   target='c', width=width, supp=2, zmin=2)
    pats.sort(key=lambda x: (len(x[0]),x[1]))
    with open(join('data','pat_all.txt'), 'w') as out:
        for p,c in reversed(pats):  # write all found patterns
            out.write(str(sorted(p)) +' : '+str(c)+'\n')
    upcs = dict(); fpcs = dict(); rpcs = dict()
    fltd = []                   # generate and mine spike trains
    for p,c in pats:            # and traverse the found patterns
        z = len(p); s = (z,c)   # create the pattern signature
        if s in upcs: upcs[s] += 1 # count unfiltered patterns
        else:         upcs[s]  = 1 # by their signatures
        if c < border[z]: continue # collect non-excluded patterns
        fltd.append((frozenset(p), c))
        if s in fpcs: fpcs[s] += 1 # count filtered patterns
        else:         fpcs[s]  = 1 # by their signatures
    fltx.sort(key=lambda x: (len(x[0]),x[1]))
    pats = patred(fltx, redids[rfnid], border, True)
    for p,c in pats:            # traverse the reduced patterns
        z = len(p); s = (z,c)   # count signatures of reduced patterns
        if s in rpcs: rpcs[s] += 1
        else:         rpcs[s]  = 1
    fltd.sort(key=lambda x: (len(x[0]),x[1]))
    with open(join('data','pat_flt.txt'), 'w') as out:
        for p,c in reversed(fltd):  # write filtered and sorted patterns
            out.write(str(sorted(p)) +' : '+str(c)+'\n')
    pats.sort(key=lambda x: (len(x[0]),x[1]))
    with open(join('data','pat_red.txt'), 'w') as out:
        for p,c in reversed(pats):  # write reduced and sorted patterns
            out.write(str(sorted(p)) +' : '+str(c)+'\n')
    with open(join('data','pcs_all.txt'), 'w') as out:
        for x in sorted([(s[0], s[1], upcs[s]) for s in upcs]):
            out.write('%g %g %g\n' % x)
    with open(join('data','pcs_flt.txt'), 'w') as out:
        for x in sorted([(s[0], s[1], fpcs[s]) for s in fpcs]):
            out.write('%g %g %g\n' % x)
    with open(join('data','pcs_red.txt'), 'w') as out:
        for x in sorted([(s[0], s[1], rpcs[s]) for s in rpcs]):
            out.write('%g %g %g\n' % x)
    return (fltd,pats)          # return the filtered/reduced patterns

#-----------------------------------------------------------------------

if __name__ == '__main__':
    # 2666 3128 6149 7067 7636 8376 9920 *10776 10922 11742 14073
    # *15761 17893 18784 19064 20043 20462 21866 21950 23207 24439
    # 24641 25316 25702 26145 *26620 28167 28807 28921 29223 29758
    # *29970 32870 33445 35303 *35506 35807 37867 38382 39058 41151
    # 48882 49372 49448 50559 51213 51748 51828 53182 53711 54127
    # 54244 54446 54498 56384 57581 >*58294 58768 59083 *60574 63639
    # 64138 *65595 65765 68848 73197 73592 74465 74520 76428 76730
    # 77869 *78510 83190 83668 *83878 85933 86017 89156 89342 89579
    # 90340 92902 *93137 94517 96355 97172 97522 97717 *97965 97982
    # 98266
    r = set()                   # init. set of condition flag results
    s = int(argv[1]) if len(argv) > 1 else 58294
    for i in range(s,100000):   # search for a proper data set
        seed(i)                 # init. the random number generator
        fltd,pats = single(100, 3, 20, 7, 7, 0.003, 0.003, 5)
        x = 0                   # check patterns for specific properties
        for p,c in fltd:        # (for proper demonstration)
            if   len(p) >= 9:            x |=  1
            elif len(p) >= 8 and c >= 4: x |=  2
            elif len(p) >= 8 and c >= 3: x |=  4
            elif len(p) <  7 and c <  7: x |=  8
            elif len(p) <= 3 and c >= 9: x |= 16
            #elif len(p) <= 4 and c >= 9: x |= 16
            elif len(p) <= 4 and c >= 8: x |= 32
        r.add(x)                # check desired conditions
        stderr.write('%6d ' % i)
        stderr.write(str(sorted(r)))
        stderr.write('\r')      # print what was found so far
        if x >= 63: break       # check desired conditions
    stderr.write('\a\n')        # terminate the output
    if x >= 63:
        print(i)
        print('filtered')
        for p,c in fltd: print(sorted(p),c)
        print('reduced')
        for p,c in pats: print(sorted(p),c)
